program ExtractSage2;

{This is a complete Delphi program for extracting sage 2 data from the
binary}
{species and index files.}
{Provided by Greg Bodecker, NIWA, 15 October 2001.}

{$APPTYPE CONSOLE}

uses
  SysUtils;

type
  {The _spec_ files contain N of the TSage2Rec data records. Details from}
  {specinfo.f90 were used to create the Delphi version of these records: }
  TSage2Rec=record
    SubTangentAlt,
    SubTangentLat,
    SubTangentLong:array[1..8] of single;
    NMCPress,
    NMCTemp,
    NMCDensity:array[1..140] of single; {0-70 km}
    NMCDensityError:array[1..140] of word; {0-70 km}
    TropopauseHeight:single;
    ChannelWavelength:array[1..7] of single;
    OzoneProfile:array[1..140] of single; {0-70km}
    NO2Profile,
    H2OProfile:array[1..100] of single; {0-50 km}
    ExtinctionAt385nm,
    ExtinctionAt453nm,
    ExtinctionAt525nm,
    ExtinctionAt1020nm:array[1..80] of single; {0-40 km}
    Density:array[1..140] of single; {0-70 km}
    AerosolSurfaceAreaDensity,
    AerosolEffectiveRadius:array[1..80] of single; {0-40 km}
    MiddleAtmosphereDensity:array[1..70] of single; {40-75 km}
    OzoneProfileError:array[1..140] of word; {0-70km}
    NO2ProfileError,
    H2OProfileError:array[1..100] of word; {0-50 km}
    ExtinctionAt385nmError,
    ExtinctionAt453nmError,
    ExtinctionAt525nmError,
    ExtinctionAt1020nmError:array[1..80] of word; {0-40 km}
    DensityError:array[1..140] of word; {0-70 km}
    AerosolSurfaceAreaDensityError,
    AerosolEffectiveRadiusError:array[1..80] of word; {0-40 km}
    MiddleAtmosphereDensityError:array[1..70] of word; {40-75 km}
    BitFlag:array[1..140] of word;
  end;

  {The _index_ file contains 1 TSage2IndexRec record which provides index}
  {information on the N records in the species file. Details from        }
  {indexinfo.f90 were used to create the Delphi version of these records:}
  TSage2IndexRec=record
    NumProf,
    ModelRevisionDate:cardinal;
    DriverVersion,
    TransmissionVersion,
    InversionVersion,
    SpectroscopyVersion:array[0..7] of char;
    EphemerisFilename,
    MetFilename,
    RefractionFilename,
    TransmissionFilename,
    SpeciesProfileFilename:array[0..31] of char;
    FillValue,
    GridSize:single;
    GeometricAltitudes:array[1..200] of single;
    MiddleAtmosphereAltitudes:array[1..70] of single;
    TransMissionRange,
    OzoneRange,
    NO2Range,
    H2ORange,
    AerosolExtinctionRange,
    DensityRange,
    AerosolSurfaceAreaRange:array[1..2] of single;
    EventDates,
    EventNumber,
    EventTime:array[1..930] of cardinal;
    DecimalDay,
    SubTangentLatitudeAt20km,
    SubTangentLongitudeAt20km,
    SpacecraftBeta,
    EventDuration:array[1..930] of single;
    InstrEventType,
    LocalEventType:array[1..930] of word; {0=sr, 1=ss}
    EventDropped,
    ProcessingBitFlags,
    EphemerisRecCreationDate,
    EphemerisRecCreationTime,
    MetRecCreationDate,
    MetRecCreationTime,
    RefractionRecCreationDate,
    RefractionRecCreationTime,
    TransmissionRecCreationDate,
    TransmissionRecCreationTime,
    SpeciesRecCreationDate,
    SpeciesRecCreationTime:array[1..930] of cardinal;
  end;

var
  Source:file of TSage2Rec;
  IndexSource:file of TSage2IndexRec;
  Data:TSage2Rec;
  IndexData:TSage2IndexRec;
  Dest:textfile;
  I:word;
begin
  {Reading in the first species record from a sage II data file:}
  assignfile(Source,'c:\sage2\v6.1\sage_II_spec_200001.6.10');
  reset(Source);
  read(Source,Data);
  closefile(Source);

  {To read in e.g. the 55th record from a sage II data file:}
  assignfile(Source,'c:\sage2\v6.1\sage_II_spec_200001.6.10');
  reset(Source);
  seek(Source,54*sizeof(TSage2Rec));
  read(Source,Data);
  closefile(Source);

  {Writing out the ozone profile from this first record:}
  assignfile(Dest,'c:\ozone.dat');
  rewrite(Dest);
  for I:=1 to 140 do writeln(Dest,I*0.5:5:1,' ',Data.OzoneProfile[I]:10:5);
  closefile(Dest);

  {Reading in the first index record from a sage II index file.}
  assignfile(IndexSource,'c:\sage2\v6.1\sage_II_index_200001.6.10');
  reset(IndexSource);
  read(IndexSource,IndexData);
  closefile(IndexSource);

  {Writing out the event times for all of the records on the species file:}
  assignfile(Dest,'c:\test.dat');
  rewrite(Dest);
  for I:=1 to 930 do writeln(Dest,I:3,' ',IndexData.EventTime[I]:6);
  closefile(Dest);
end.