;+
;example calls
;bits = bitinfo('SPEC',spec.infvec)
;bits = bitinfo('INDEX',ind.infvec)
;-
function bitinfo,typ,infvec,quiet=quiet
indbits = {INDINFVEC,$
           PMC_Present:               0, $;0
           H2O_Zero_Found:            0, $;1
           H2O_Slow_Convergence:      0, $;2
           H2O_EGA_Failure:           0, $;3
           spare_bit_4:               0, $;4
           spare_bit_5:               0, $;5
           spare_bit_6:               0, $;6
           spare_bit_7:               0, $;7
           Incomplete_NMC_Data:       0, $;8
           spare_bit_9:               0, $;9
           spare_bit_10:              0, $;10
           spare_bit_11:              0, $;11
           spare_bit_12:              0, $;12
           spare_bit_13:              0, $;13
           spare_bit_14:              0, $;14
           Mirror_Model:              0, $;15
           spare_bit_16:              0, $;16
           spare_bit_17:              0, $;17
           spare_bit_18:              0, $;18
           Twomey_Non_Conv_Rayleigh:  0, $;19
           Twomey_Non_Conv_386_Aero:  0, $;20
           Twomey_Non_Conv_452_Aero:  0, $;21
           Twomey_Non_Conv_525_Aero:  0, $;22
           Twomey_Non_Conv_1020_Aero: 0, $;23
           Twomey_Non_Conv_NO2:       0, $;24
           Twomey_Non_Conv_Ozone:     0, $;25
           spare_bit_26:              0, $;26
           spare_bit_27:              0, $;27
           spare_bit_28:              0, $;28
           spare_bit_29:              0, $;29
           No_Shock_Correction:       0}  ;30

specbits = {SPECINFVEC,$
            Separation_Method:        0, $;bits 0-2
            filler_bit_1:             0, $;(1)
            filler_bit_2:             0, $;(2)
            One_Chan_Aerosol_Corr:    0, $;3
            No_935_Aerosol_Corr:      0, $;4
            Reserved_for_Trans:       0, $;5
            NO2_Extrap:               0, $;6
            spare_bit_7:              0, $;7
            Large_525_OD:             0, $;8
            Large_1020_OD:            0, $;9
            spare_bit_10:             0, $;10
            spare_bit_11:             0, $;11
            No_H2O_Corr:              0, $;12
            spare_bit_13:             0, $;13
            In_Troposphere:           0}  ;14


if typ eq 'INDEX' then begin
    bitstruct = replicate(indbits,n_elements(infvec))
    btbg = 0
    bten = n_tags(bitstruct)-1
endif else if typ eq 'SPEC' then begin
    bitstruct = replicate(specbits,n_elements(infvec))
    btbg = 2
    bten = n_tags(bitstruct)-1
endif else begin
    if not keyword_set(quiet) then begin
        print,'structure type not provided or not recognized.'
    endif
    return,-1
endelse

;in case of species structure, combine the first three bits into one
;value
if typ eq 'SPEC' then begin
    fill = where(infvec eq -999)
    if fill(0) ne -1 then begin
        bitstruct(fill).separation_method = -999
    endif 
    gd = where(infvec ne -999)
    if gd(0) ne -1 then begin
        bitstruct(gd).separation_method = infvec(gd) mod 8
    endif
endif
;copy the rest of the bits one bit at a time
for i=btbg,bten do begin
    fill = where(infvec eq -999)
    if fill(0) ne -1 then begin
        bitstruct(fill).(i) = -999
    endif
    gd = where(infvec ne -999)
    if gd(0) ne -1 then begin
        bitstruct(gd).(i) = bit(infvec(gd),i)
    endif
endfor

;get the shape of the data structure
shape = size(infvec)
;if it's a 2-d array, return a 2-d array of structures
if (shape(0) eq 2) then begin
    bitstruct = reform(bitstruct,shape(1),shape(2))
endif else if (shape(0) gt 2) then begin
    print,'code not set up to hold 3 or more dimensional arrays!'
endif

return,bitstruct
end


