;+
;example calls
;bits = bitinfo('SPEC',spec.infvec)
;bits = bitinfo('INDEX',ind.infvec)
;-
function bitinfo_610,typ,infvec,quiet=quiet
indbits = {INDINFVEC_610,$
           PMC_Present:               0, $;0 Polar Mesospheric Cloud(PMC) found in profile between 70 and 90km
           H2O_Zero_Found:            0, $;1 Zero or negative mixing ratio inferred
           H2O_Slow_Convergence:      0, $;2 Water vapor retrieval required more than X iterations
           H2O_EGA_Failure:           0, $;3 Emmisivity Curve-of-Growth Approximation(EGA) tool failure
           default_nmc_temp_errors:   0, $;4 Temp error values from model instead of NCEP reported measurment errors
           spare_bit_5:               0, $;5
           spare_bit_6:               0, $;6
           spare_bit_7:               0, $;7
           Incomplete_NMC_Data:       0, $;8 One or more mandatory levels were missing from National Center for Environmental Prediction(NCEP) data
           spare_bit_9:               0, $;9
           spare_bit_10:              0, $;10
           spare_bit_11:              0, $;11
           spare_bit_12:              0, $;12
           spare_bit_13:              0, $;13
           spare_bit_14:              0, $;14
           Mirror_Model:              0, $;15 Mirror reflectivity is modeled;insufficient high altitude data
           spare_bit_16:              0, $;16
           spare_bit_17:              0, $;17
           spare_bit_18:              0, $;18
           Twomey_Non_Conv_Rayleigh:  0, $;19 Twomey-Chahine (T-C) inversion routine failure for Rayleigh retrieval 
           Twomey_Non_Conv_386_Aero:  0, $;20 T-C inversion routine failure for 386-nm aerosol extinction retrieval
           Twomey_Non_Conv_452_Aero:  0, $;21 T-C inversion routine failure for 452-nm aerosol extinction retrieval
           Twomey_Non_Conv_525_Aero:  0, $;22 T-C inversion routine failure for 525-nm aerosol extinction retrieval
           Twomey_Non_Conv_1020_Aero: 0, $;23 T-C inversion routine failure for  1020-nm aerosol extinction retrieval
           Twomey_Non_Conv_NO2:       0, $;24 T-C inversion routine failure for NO2 retrieval 
           Twomey_Non_Conv_Ozone:     0, $;25 T-C inversion routine failure for ozone retrieval 
           spare_bit_26:              0, $;26
           spare_bit_27:              0, $;27
           spare_bit_28:              0, $;28
           spare_bit_29:              0, $;29
           No_Shock_Correction:       0}  ;30 No correction for the electronic transient was performed; usually a short event with too few extraterrestrial solar irradiance (Io) available 

specbits = {SPECINFVEC_610,$
            Separation_Method:        0, $;bits 0-2 Method used to separate between ozone, NO2, and aerosol

; Species Separation Method 
; no_aerosol_method (value 0) Four channels used (3-6); Ozone, NO2, no aerosol inferred 
; trans_no_aero_to_five_chan (value 1) Transition 
; standard_method (value 2) Five channels used (1,3-6); Ozone, NO2, and aerosol (3) 
; trans_five_chan_to_low (value 3) Transition 
; four_chan_method (value 4) Four channels used (1,3-5); Ozone, aerosol (3)
; trans_four_chan_to_three_chan (value 5) Transition 
; three_chan_method (value 6) Three channels used (1,3,4); Ozone, aerosol (2) 
; extension_method (value 7) Channel 1 only, aerosol (1)
 
            filler_bit_1:             0, $;(1)
            filler_bit_2:             0, $;(2)
            One_Chan_Aerosol_Corr:    0, $;3 Aerosol correction based on 1020-nm aerosol only 
            No_935_Aerosol_Corr:      0, $;4 No aerosol correction in the water vapor retrieval (based on the 935-nm channel)
            Reserved_for_Trans:       0, $;5
            NO2_Extrap:               0, $;6 Relevant to water vapor retrieval NO2 is extrapolated at this altitude based on a vertical profile that terminated at a higher altitude
            Water_vapor_ratio:        0, $;7-10 Ratio of the water vapor slant path optical depth to the total optical depth in 940nm
            filler_bit_8:             0, $;(8)
            filler_bit_9:             0, $;(9)
            filler_bit_10:            0, $;(10)
            Large_525_OD:             0, $;11 525-nm aerosol slant path optical depth is large at or above this level and may influence ozone retrieval 
            Large_1020_OD:            0, $;12 1020-nm aerosol slant path optical depth is large at or above this level and may influence ozone retrieval 
            No_H2O_Corr:              0, $;13 Ozone not corrected for water vapor 
            In_Troposphere:           0}  ;14 Altitude is below the NCEP-provided tropopause altitude 


if typ eq 'INDEX' then begin
    bitstruct = replicate(indbits,n_elements(infvec))
    bten = n_tags(bitstruct)-1
endif else if typ eq 'SPEC' then begin
    bitstruct = replicate(specbits,n_elements(infvec))
    bten = n_tags(bitstruct)-1
endif else begin
    if not keyword_set(quiet) then begin
        print,'structure type not provided or not recognized.'
    endif
    return,-1
endelse

;copy the bits one bit at a time
for i=0,bten do begin
    fill = where(infvec eq -999)
    if fill(0) ne -1 then begin
        bitstruct(fill).(i) = -999
    endif
    gd = where(infvec ne -999)
    if gd(0) ne -1 then begin
        bitstruct(gd).(i) = bit(infvec(gd),i)
    endif
endfor
;in case of species structure, combine the first three bits into one
;value
if typ eq 'SPEC' then begin
    gd = where(infvec ne -999)
    if gd(0) ne -1 then begin
        bitstruct(gd).separation_method = infvec(gd) mod 8
;also combine bits 7 to 10 into one value
        bitstruct(gd).water_vapor_ratio = ishft(infvec(gd),-7) mod 16 
    endif
;set all the filler bits (1-2 and 8-10) to fill
    bitstruct.filler_bit_1 = -999
    bitstruct.filler_bit_2 = -999
    bitstruct.filler_bit_8 = -999
    bitstruct.filler_bit_9 = -999
    bitstruct.filler_bit_10 = -999
endif

;get the shape of the data structure
shape = size(infvec)
;if it's a 2-d array, return a 2-d array of structures
if (shape(0) eq 2) then begin
    bitstruct = reform(bitstruct,shape(1),shape(2))
endif else if (shape(0) gt 2) then begin
    print,'code not set up to hold 3 or more dimensional arrays!'
endif

return,bitstruct
end




