;+
;ALTS and DATA should be 2-dimensional arrays of the same shape,
;where the second index loops over events and the first loops over
;levels within an event.  Each event should have the same number of
;levels but they don't have to be at the same altitudes. Alternately,
;the altitude array can be a 1-dimensional array if all events are on
;the same altitude grid.  
;Examples: an array of g2p items: alts=g2p(0:nev).geoalt, data=g2p(0:nev).o3nd
;          from a data file: alts=alldat(0,1:70,*),data=alldat(1,1:70,*)
;          from the new structure: alts=findgen(200)*0.5+0.5,
;                                  data=trans(0:nev).spod1020
;MEANPROF is the output mean profile, TARGET(input) is the
;corresponding altitude array.
;NOTE: if the fill keyword is not set, fill is assumed to be -999.
;
;$Header: /home/users/burton/sageprof-rcs/meanprof.pro,v 1.7 2003/10/06 14:40:27 burton Exp $
;-

pro meanprof,alts,data,target,meanprof,count,stddev,fill=fill

nd = n_elements(data)
na = n_elements(alts)
onedim = 0
if na ne nd then begin
    if nd/na eq nd/float(na) then begin
        onedim = 1
    endif else begin
        print,'MEANPROF:The sizes of alts and data arrays are not compatible'
        help,alts,data
        retall
    endelse
endif

if n_elements(fill) lt 1 then fill = -999.d0
meanprof = target*0.d0+fill
count = fix(target*0)
stddev = target*0.d0+fill

for ilev = 0,n_elements(target)-1 do begin
    mtc = where(alts eq target(ilev))
    if mtc(0) ne -1 then begin
        if onedim then begin
            mtc = mtc(0)
            gd = where(data(mtc,*) ne fill and finite(data(mtc,*)),ngd)
            if gd(0) ne -1 then gddata = data(mtc,gd)
        endif else begin
            gd = where(data(mtc) ne fill and finite(data(mtc)),ngd)
            if gd(0) ne -1 then gddata = data(mtc(gd))
        endelse
        if gd(0) ne -1 then begin
            count(ilev) = ngd
            meanprof(ilev) = total( gddata )/ngd
            stddev(ilev) = sqrt( total( (gddata - $
                                         meanprof(ilev))^2 )/ngd ) 
        endif
    endif
endfor

end
