;+
;ALTS and DATA should be 2-dimensional arrays of the same shape,
;where the second index loops over events and the first loops over
;levels within an event.  Each event should have the same number of
;levels but they don't have to be at the same altitudes. Alternately,
;the altitude array can be a 1-dimensional array if all events are on
;the same altitude grid.  
;Examples: an array of g2p items: alts=g2p(0:nev).geoalt, data=g2p(0:nev).o3nd
;          from a data file: alts=alldat(0,1:70,*),data=alldat(1,1:70,*)
;          from the new structure: alts=findgen(200)*0.5+0.5,
;                                  data=trans(0:nev).spod1020
;MEDIANPROF is the output median profile, TARGET(input) is the
;corresponding altitude array.
;NOTE: if the fill keyword is not set, fill is assumed to be -999.
;
;$Header: /g2prof1/readers/medianprof.pro,v 1.1 2001/06/25 14:28:42 burton Exp $
;-

pro medianprof,alts,data,target,medianprof,count,fill=fill

nd = n_elements(data)
na = n_elements(alts)
onedim = 0
if na ne nd then begin
    if nd/na eq nd/float(na) then begin
        onedim = 1
    endif else begin
        print,'MEDIANPROF:The sizes of alts and data arrays are not compatible'
        help,alts,data
        retall
    endelse
endif

if n_elements(fill) lt 1 then fill = -999.d0
medianprof = target*0.d0+fill
count = fix(target*0)

for ilev = 0,n_elements(target)-1 do begin
    mtc = where(alts eq target(ilev))
    if mtc(0) ne -1 then begin
        if onedim then begin
;in this case, mtc is just one value, and should be used as the first index
            mtc = mtc(0)
            gd = where(data(mtc,*) ne fill,ngd)
            if gd(0) ne -1 then gddata = data(mtc,gd)
        endif else begin
;in this case, mtc represents all the data at the altitude of interest
            gd = where(data(mtc) ne fill,ngd)
            if gd(0) ne -1 then gddata = data(mtc(gd))
        endelse
        if gd(0) ne -1 then begin
            count(ilev) = ngd
            medianprof(ilev) = median( gddata )
        endif
    endif
endfor

end
