;
; IDL Event Callback Procedures
; sagebrowser_eventcb
;
; Generated on:	07/09/2002 09:20.06
;
;-----------------------------------------------------------------
; Activate Button Callback Procedure.
; Argument:
;   Event structure:
;
;   {WIDGET_BUTTON, ID:0L, TOP:0L, HANDLER:0L, SELECT:0}
;
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;   SELECT is set to 1 if the button was set, and 0 if released.
;       Normal buttons do not generate events when released, so
;       SELECT will always be 1. However, toggle buttons (created by
;       parenting a button to an exclusive or non-exclusive base)
;       return separate events for the set and release actions.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
pro OnExit, Event
close,/all
WIDGET_CONTROL, Event.top, /DESTROY
Return

end
;-----------------------------------------------------------------
; Activate Button Callback Procedure.
; Argument:
;   Event structure:
;
;   {WIDGET_BUTTON, ID:0L, TOP:0L, HANDLER:0L, SELECT:0}
;
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;   SELECT is set to 1 if the button was set, and 0 if released.
;       Normal buttons do not generate events when released, so
;       SELECT will always be 1. However, toggle buttons (created by
;       parenting a button to an exclusive or non-exclusive base)
;       return separate events for the set and release actions.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
pro plotdata, Event
	common Events, etype
	indexinfo_610,index610
	specinfo_610,spec610

	ext = 0.
	speclabels=['1020-nm Extinction (1/km)','525-nm Extinction (1/km)','452-nm Extinction (1/km)', $
		'386-nm Extinction (1/km)','Ozone (cm!E-3!N)','Nitrogen Dioxide (cm!E-3!N)','Water Vapor (ppm)','Extinction Ratio']
;
; Get the values for day, month, year, species, minimum and maximum altitude, and plotting range
;
	id_day=widget_info(Event.top, FIND_BY_UNAME='Day')
	id_year=widget_info(Event.top, FIND_BY_UNAME='Year')
	id_month=widget_info(Event.top, FIND_BY_UNAME='Month')
	id_sunrise=widget_info(Event.top, FIND_BY_UNAME='Sunrise')
	id_sunset=widget_info(Event.top, FIND_BY_UNAME='Sunset')
        id_version=widget_info(Event.top, FIND_BY_UNAME='Version')
	id_species=widget_info(Event.top, FIND_BY_UNAME='Species')

	day = widget_info(id_day, /DropList_Select)
	month = widget_info(id_month, /DropList_Select)
	year = widget_info(id_year, /DropList_Select)
        xversion = widget_info(id_version, /DropList_Select)
	xspec = widget_info(id_species, /DropList_Select)

	id_minalt =  widget_info(Event.top, FIND_BY_UNAME='MinAlt')
	id_maxalt =  widget_info(Event.top, FIND_BY_UNAME='MaxAlt')
	widget_control,id_maxalt,Get_Value=maxalt
	widget_control,id_minalt,Get_Value=minalt
;
; Adjust minalt and related parameters to reflect that '0' is really 0.5 km
;
	if(minalt eq 0) then begin
		vcount = 2*maxalt
		zmin = 0
		zmax = maxalt*2-1
	endif else begin
		vcount = 2*(maxalt-minalt) + 1
		zmin = minalt*2-1
		zmax = maxalt*2-1
	endelse

	id_minval =  widget_info(Event.top, FIND_BY_UNAME='MinVal')
	id_maxval =  widget_info(Event.top, FIND_BY_UNAME='MaxVal')
	widget_control,id_maxval,Get_Value=maxval
	widget_control,id_minval,Get_Value=minval
;
; Check for data filtering
;
	id_filter = widget_info(event.top, Find_By_UName='Filter')
	xfilter = widget_info(id_filter, /DropList_Select)
	filtervals = [30000.,1000,2500,5000.,10000.,20000.,30000.]
;
; convert day, month, and year from menu position to date values
;
	day  = day + 1
	month = month + 1
	year = year + 1984
	yyyymmdd = year*10000+month*100+day
;
; determine SAGE II data file names
;
        setshortcuts,shortcuts
        ns = widget_info(id_version,/droplist_number)
        if xversion eq ns-1 then begin
; Browse is set; use the directory & filename chosen previously
            version = !_versdefault 
        endif else begin
            version = shortcuts(ns-2-xversion).path
        endelse
        sage_filenames,spec,index,version=version,yyyymm=year*100+month
;
; Open index files and find events that match required values
;
	close,/all
	openr,1,index,error=err
	if(err ne 0) then begin
		Erase
		sout = string(format='(''Error opening file: '',a)',index)
		xyouts,0.2,0.5,sout,/norm
		print,'stop'
		Return
	Endif
	readu,1,index610
	s = where(index610.type_sat eq etype and index610.yyyymmdd eq yyyymmdd and index610.dropped eq 0, ecount)
	if (ecount lt 10) then begin
		Erase
		sout = string(format='(''Insufficient Data to plot. Number of Events:'',i2)',ecount)
		xyouts,0.2,0.5,sout,/norm
		print,'stop'
		Return
	Endif
;
; Read events from the species files
;
	close,/all
	idx = n_elements(s)
	openr,1,spec,error=err
	if(err ne 0) then begin
		Erase
		sout = string(format='(''Error opening file: '',a)',index)
		xyouts,0.2,0.5,sout,/norm
		print,'stop'
		Return
	Endif
	xdata = assoc(1,spec610)
;
; Pull out the requested data type over the range requested
;
	for k = 0, idx-1 do begin
		ydata = xdata[s(k)]
		if(k eq 0) then begin
			if( xspec eq 0) then begin
				ext = ydata.ext1020(zmin:zmax)
				sq = where(ydata.ext1020_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext(sq) = -999.
			endif
			if( xspec eq 1) then begin
				ext = ydata.ext525(zmin:zmax)
				sq = where(ydata.ext525_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext(sq) = -999.
			endif
			if( xspec eq 2) then begin
				ext = ydata.ext452(zmin:zmax)
				sq = where(ydata.ext452_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext(sq) = -999.
			endif
			if( xspec eq 3) then begin
				ext = ydata.ext386(zmin:zmax)
				sq = where(ydata.ext386_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext(sq) = -999.
			endif
			if( xspec eq 4) then begin
				ext = ydata.O3(zmin:zmax)
				sq = where(ydata.O3_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext(sq) = -999.
			endif
			if( xspec eq 5) then begin
				ext = ydata.NO2(zmin:zmax)
				sq = where(ydata.NO2_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext(sq) = -999.
			endif
			if( xspec eq 6) then begin
				ext = ydata.H2O(zmin:zmax)
				sq = where(ydata.H2O_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext(sq) = -999.
			endif
			if( xspec eq 7) then begin
				ext = ydata.ext525(zmin:zmax)/ydata.ext1020(zmin:zmax)
				sx = where(ydata.ext525_err(zmin:zmax) gt filtervals(xfilter) $
					    or ydata.ext1020_err(zmin:zmax) gt filtervals(xfilter))
				if(sx(0) ne -1) then ext(sx) = -999.
			endif
			lon = index610.lon(s(k))
			lat = index610.lat(s(k))
		endif else begin
			if( xspec eq 0) then begin
				ext0 = ydata.ext1020(zmin:zmax)
				sq = where(ydata.ext1020_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext0(sq) = -999.
				ext = [ext,ext0]
			endif
			if( xspec eq 1) then begin
				ext0 = ydata.ext525(zmin:zmax)
				sq = where(ydata.ext525_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext0(sq) = -999.
				ext = [ext,ext0]
			endif
			if( xspec eq 2) then begin
				ext0 = ydata.ext452(zmin:zmax)
				sq = where(ydata.ext452_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext0(sq) = -999.
				ext = [ext,ext0]
			endif
			if( xspec eq 3) then begin
				ext0 = ydata.ext386(zmin:zmax)
				sq = where(ydata.ext386_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext0(sq) = -999.
				ext = [ext,ext0]
			endif
			if( xspec eq 4) then begin
				ext0 = ydata.O3(zmin:zmax)
				sq = where(ydata.O3_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext0(sq) = -999.
				ext = [ext,ext0]
			endif
			if( xspec eq 5) then begin
				ext0 = ydata.NO2(zmin:zmax)
				sq = where(ydata.NO2_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext0(sq) = -999.
				ext = [ext,ext0]
			endif
			if( xspec eq 6) then begin
				ext0 = ydata.H2O(zmin:zmax)
				sq = where(ydata.H2O_err(zmin:zmax) gt filtervals(xfilter),icount)
				if(icount ge 1) then ext0(sq) = -999.
				ext = [ext,ext0]
			endif
			if( xspec eq 7) then begin
				ext0 = ydata.ext525(zmin:zmax)/ydata.ext1020(zmin:zmax)
				sx = where(ydata.ext525_err(zmin:zmax) gt filtervals(xfilter) $
				        or ydata.ext1020_err(zmin:zmax) gt filtervals(xfilter))
				if(sx(0) ne -1) then ext0(sx) = -999.
				ext = [ext,ext0]
			endif
			lon = [lon,index610.lon(s(k))]
			lat = [lat,index610.lat(s(k))]
		endelse
            endfor
	;
	;fill gaps with missing data
	;
	s = sort(lon)
	xlon = lon(s)
	xlat = lat(s)
	igap = fltarr(14)
	for k = 0, idx -2 do igap(k) = Round((xlon(k+1)-xlon(k))/24.) - 1
	mgap = max(igap)
	s = where(igap ge 1.,ngap)
;	stop
	if(mgap gt 0) then begin
		missing = fltarr(zmax-zmin+1) - 999.
		for j = 0, ngap-1 do begin
			jgap = 	s(j)
			dlon = (xlon(jgap+1)-xlon(jgap))/float(igap(s(j))+1.)
			for k = 0, igap(s(j))-1 do begin
				tlon = xlon(jgap)+dlon*float(k+1)
				if(tlon lt -180.) then tlon = -180.-tlon
				if(tlon gt 180.) then tlon = tlon - 360.
				lon = [lon, tlon]
				lat = [lat, xlat(ngap)]
				ext = [ext, missing]
			endfor
		endfor
	endif

	idx = idx + total(igap>0)
;
; resort the array into longitude order
;
	speclab = speclabels(xspec)
	ext=reform(ext,vcount,idx)
	if(minalt eq 0) then alt = findgen(vcount)*0.5 + 0.5
	if(minalt gt 0) then alt = findgen(vcount)*0.5 + minalt
	latlab = string(format='(''Lats.: '',f5.1,'' to '',f5.1)',min(lat),max(lat))
	s = sort(lon)
	lon = lon(s)
	;print,lon
	a = fltarr(idx,vcount)
	for k = 0, idx-1 do a(k,0:vcount-1) = ext(0:vcount-1,s(k))
;
; Remove 1020-nm extinction post cloud 'tail'
;
	if(xspec eq 0) then begin
		for k = 0, idx-1 do begin
			iflag = 0
			For n = vcount-1, 0, -1 do begin
				if(a(k,n) gt 0.02 or iflag eq 1) then begin
					a(k,n) = !values.f_nan
					iflag = 1
				endif
			endfor
		endfor
	EndIf
	s = where(lon gt -200)
;
; Set the plot parameters
;
	altmin = max([minalt,0.5])
	!x.style=1
	!y.style=1
	!x.range=[min(lon(s)),max(lon)]
	!y.range=[altmin,maxalt]
	!x.title='Longitude'
	!y.title='Altitude (km)'
	!p.charsize = 2.0

	id_draw=widget_info(Event.top, FIND_BY_UNAME='Draw')
;Obtain the window index.
	WIDGET_CONTROL, id_draw, GET_VALUE = index
;Set the new widget to be the current graphics window
	WSET, index
;
; Determine the request color index and remap
	id_ct=widget_info(Event.top, FIND_BY_UNAME='Colors')
	ct = widget_info(id_ct, /DropList_Select)
	if(ct eq 0) then begin
		wowct
	endif else begin
		loadct,ct-1
	endelse
	plot,[0],/nodata,font=1
;
; Determine plot size
;
	py0 = !p.clip(1)
	py0 = long(round(py0))
	py1 = !p.clip(3)
	py1 = long(round(py1))

	px0 = !p.clip(0)
	px0 = long(round(px0))
	px1 = !p.clip(2)
	px1 = long(round(px1))

	dpx = px1-px0
	dpy = py1-py0
;
; Scale the species values to 1 to 254 range and expand to size of plot area
;
	leg_min = minval
	leg_max = maxval
	clevel = findgen(2*(maxval-minval)+1)*0.5+minval
	;print,clevel
	;stop
	slope = 253./(leg_max-leg_min)
	if(xspec le 6) then begin
		ext = 1. + slope*( alog10(a) - leg_min )
	endif else begin
		ext = 1. + slope*(a - leg_min)
        endelse
	s = where(ext lt 1.)
	If(s(0) ne -1) then begin
		ext(s) = !values.f_nan
		a(s) = !values.f_nan
        Endif
	s = where(ext gt 254.)
	If(s(0) ne -1) then ext(s) = 254.
;NOTE, the following line takes a long time.
        id_Fast = widget_info(Event.top, FIND_BY_UNAME='Fast')
        widget_control,id_Fast,Get_UValue=xFast
        if xFast eq 1 then begin
            w = congrid(ext,px1-px0,py1-py0)
        endif else begin
            expand,ext,px1-px0,py1-py0,w,maxval=254.,fillval=255.
        endelse
;
; Plot the array as a bit map overlayed with contours
;
        if !d.n_colors ge 256. then begin
            tv,w,px0,py0,xsize=px1-px0,ysize=py1-py0
        endif else begin
            tvscl,w,px0,py0,xsize=px1-px0,ysize=py1-py0,/nan
        endelse
	s = where(lon gt -200)
	if(xspec le 6) then contour,alog10(a),lon,alt,max_value=maxval+1,levels=clevel, $
		c_labels=[1,0,1,0,1,0,1,0,1,0],/overplot,font=0
	if(xspec eq 7) then contour,a,lon,alt,max_value=maxval+1,levels=clevel, $
		c_labels=[1,0,1,0,1,0,1,0,1,0],/overplot,font=0
	lalt = (maxalt-minalt)*1.01 + minalt
;	xyouts,0.55,0.97, latlab,font=1,/norm
;	xyouts,0.20,0.97, speclab,font=1,/norm
;	xyouts,0.75,0.97, string(format='(i2,''/'',i2,''/'',i4)',month,day,year),font=1,/norm
        xyouts,0.5,0.97,align=0.5,speclab+'  '+latlab+'  '+$
               string(format='(i2,''/'',i2,''/'',i4)',month,day,year),$
               font=1,/norm
	plot,[0],/nodata,/noerase,font=1
;
; Save Output in Postscript Format
;
	id_PS = widget_info(Event.top, FIND_BY_UNAME='PS_Save')
	widget_control,id_PS,Get_UValue=xPS
	if(xPS eq 1) then begin
		set_gph,/cps,/land
                fn = string(long(systime(1) mod 10000000),$
                            form='("S2DB",i8.8,".ps")')
                device,filename=fn
		altmin = max([minalt,0.5])
		!x.style=1
		!y.style=1
		!x.range=[min(lon(s)),max(lon)]
		!y.range=[altmin,maxalt]
		!x.title='Longitude'
		!y.title='Altitude (km)'
		!p.charsize = 1.5
			if(ct eq 0) then begin
			wowct
		endif else begin
			loadct,ct-1
		endelse
		plot,[0],/nodata,font=1
	;
	; Resize the arrays for PS output
	;
		py0 = !p.clip(1)
		py0 = long(round(py0))
		py1 = !p.clip(3)
		py1 = long(round(py1))

		px0 = !p.clip(0)
		px0 = long(round(px0))
		px1 = !p.clip(2)
		px1 = long(round(px1))

		dpx = px1-px0
		dpy = py1-py0
	;
	; Draw the output graphics
	;
                if !d.n_colors ge 256. then begin
                    tv,w,px0,py0,xsize=px1-px0,ysize=py1-py0
                endif else begin
                    tvscl,w,px0,py0,xsize=px1-px0,ysize=py1-py0,/nan
                endelse
		s = where(lon gt -200)
		if(xspec le 6) then contour,alog10(a),lon,alt,max_value=maxval+1,levels=clevel, $
			c_labels=[1,0,1,0,1,0,1,0,1,0],/overplot,font=0
		if(xspec eq 7) then contour,a,lon,alt,max_value=maxval+1,levels=clevel, $
			c_labels=[1,0,1,0,1,0,1,0,1,0],/overplot,font=0
		lalt = (maxalt-minalt)*1.01 + minalt
		xyouts,0.55,0.96, latlab,font=1,/norm
		xyouts,0.20,0.96, speclab,font=1,/norm
		xyouts,0.75,0.96, string(format='(i2,''/'',i2,''/'',i4)',month,day,year),font=1,/norm
		plot,[0],/nodata,/noerase,font=1
		Device,/close
;		cmd = string(long(systime(1) mod 10000000),format='(''rename idl.ps S2DB'',i8.8,''.ps'')')
;		spawn,cmd
		SET_GPH,/WIN
            Endif
	return
end
;
; Empty stub procedure used for autoloading.
;
pro sagebrowser_eventcb
end
;-----------------------------------------------------------------
; Handle Event Callback Function.
; Argument:
;   Event structure:
;
;   {WIDGET_<type>, ID:0L, TOP:0L, HANDLER:0L,...}
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;
;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
function Flipss, Event
	common Events, etype
	id=widget_info(Event.top, FIND_BY_UNAME='Sunset')
	widget_control,id,set_button=0
	etype=0
    return, Event ; By Default, return the event.

end
;-----------------------------------------------------------------
; Handle Event Callback Function.
; Argument:
;   Event structure:
;
;   {WIDGET_<type>, ID:0L, TOP:0L, HANDLER:0L,...}
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;
;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
function flipsr, Event
	Common Events,etype
	id=widget_info(Event.top, FIND_BY_UNAME='Sunrise')
	widget_control,id,set_button=0
	;print,id
	etype = 1
	return, Event ; By Default, return the event.

end
;-----------------------------------------------------------------
; Post Create Widget Procedure.
; Argument:
;   wWidget - ID number of specific widget.
;
;   Any keywords passed into the generated widget creation procedure
;       are passed into this procudure.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
pro deftype, wWidget, _EXTRA=_VWBExtra_
	Common Events,etype
	;id=widget_info(wWidget, FIND_BY_UNAME='Sunset')
	widget_control,wWidget,set_button=1
	etype = 1

end
;-----------------------------------------------------------------
; Slider Value Changed Callback Procedure.
; Argument:
;   Event structure:
;
;   {WIDGET_SLIDER, ID:0L, TOP:0L, HANDLER:0L, VALUE:0L, DRAG:0}
;
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;   VALUE returns the new value of the slider. DRAG returns integer 1
;       if the slider event was generated as part of a drag
;       operation, or zero if the event was generated when the user
;       had finished positioning the slider.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
pro AdjustLimits, Event

end
;-----------------------------------------------------------------
; Handle Event Callback Function.
; Argument:
;   Event structure:
;
;   {WIDGET_<type>, ID:0L, TOP:0L, HANDLER:0L,...}
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;
;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
function AdjustPara, Event
;
; Adjust plotting parameters to match requested species
;
	id_species = widget_info(Event.top, FIND_BY_UNAME='Species')
	id_minalt =  widget_info(Event.top, FIND_BY_UNAME='MinAlt')
	id_maxalt =  widget_info(Event.top, FIND_BY_UNAME='MaxAlt')
	id_minval =  widget_info(Event.top, FIND_BY_UNAME='MinVal')
	id_maxval =  widget_info(Event.top, FIND_BY_UNAME='MaxVal')
	xspec = widget_info(id_species, /DropList_Select)
	if(xspec le 3) then Begin
		widget_control,id_minalt,SET_SLIDER_Min=0
		widget_control,id_maxalt,SET_SLIDER_Max=40
		widget_control,id_minval,SET_SLIDER_Min=-7
		widget_control,id_maxval,SET_SLIDER_Max=-1
		widget_control,id_minalt,SET_SLIDER_Max=30
		widget_control,id_maxalt,SET_SLIDER_Min=10
		widget_control,id_minval,SET_SLIDER_Max=-3
		widget_control,id_maxval,SET_SLIDER_Min=-5
		widget_control,id_maxval,Set_Value=-2
		widget_control,id_maxalt,Set_Value=40
		widget_control,id_minval,Set_Value=-5
		widget_control,id_minalt,Set_Value=0
	endif
	if(xspec eq 4) then Begin ;Ozone
		widget_control,id_minalt,SET_SLIDER_Min=0
		widget_control,id_maxalt,SET_SLIDER_Max=70
		widget_control,id_minval,SET_SLIDER_Min=8
		widget_control,id_maxval,SET_SLIDER_Max=17
		widget_control,id_minalt,SET_SLIDER_Max=60
		widget_control,id_maxalt,SET_SLIDER_Min=10
		widget_control,id_minval,SET_SLIDER_Max=16
		widget_control,id_maxval,SET_SLIDER_Min=10
		widget_control,id_maxval,Set_Value=13
		widget_control,id_maxalt,Set_Value=70
		widget_control,id_minval,Set_Value=9
		widget_control,id_minalt,Set_Value=0
	endif
	if(xspec eq 5) then Begin ;NO2
		widget_control,id_minalt,SET_SLIDER_Min=0
		widget_control,id_maxalt,SET_SLIDER_Max=45
		widget_control,id_minval,SET_SLIDER_Min=7
		widget_control,id_maxval,SET_SLIDER_Max=13
		widget_control,id_minalt,SET_SLIDER_Max=35
		widget_control,id_maxalt,SET_SLIDER_Min=10
		widget_control,id_minval,SET_SLIDER_Max=13
		widget_control,id_maxval,SET_SLIDER_Min=8
		widget_control,id_maxval,Set_Value=11
		widget_control,id_maxalt,Set_Value=45
		widget_control,id_minval,Set_Value=8
		widget_control,id_minalt,Set_Value=15
	endif
	if(xspec eq 6) then Begin ;H2O
		widget_control,id_minalt,SET_SLIDER_Max=35
		widget_control,id_maxalt,SET_SLIDER_Min=10
		widget_control,id_minalt,SET_SLIDER_Min=0
		widget_control,id_maxalt,SET_SLIDER_Max=45
		widget_control,id_minval,SET_SLIDER_Min=-7
		widget_control,id_maxval,SET_SLIDER_Max=-2
		widget_control,id_minval,SET_SLIDER_Max=-3
		widget_control,id_maxval,SET_SLIDER_Min=-6
		widget_control,id_maxval,Set_Value=-2
		widget_control,id_maxalt,Set_Value=45
		widget_control,id_minval,Set_Value=-6
		widget_control,id_minalt,Set_Value=0
	endif
	if(xspec eq 7) then Begin ;525 to 1020 nm aerosol extinction ratio
		widget_control,id_minalt,SET_SLIDER_Max=30
		widget_control,id_minalt,SET_SLIDER_Min=0
		widget_control,id_maxalt,SET_SLIDER_Max=40
		widget_control,id_maxalt,SET_SLIDER_Min=10

		widget_control,id_minval,SET_SLIDER_Min=0
		widget_control,id_maxval,SET_SLIDER_Max=8
		widget_control,id_minval,SET_SLIDER_Max=7
		widget_control,id_maxval,SET_SLIDER_Min=1
		widget_control,id_maxval,Set_Value=5
		widget_control,id_maxalt,Set_Value=40
		widget_control,id_minval,Set_Value=1
		widget_control,id_minalt,Set_Value=0
	endif

     return, Event ; By Default, return the event.

end
;-----------------------------------------------------------------
; Slider Value Changed Callback Procedure.
; Argument:
;   Event structure:
;
;   {WIDGET_SLIDER, ID:0L, TOP:0L, HANDLER:0L, VALUE:0L, DRAG:0}
;
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;   VALUE returns the new value of the slider. DRAG returns integer 1
;       if the slider event was generated as part of a drag
;       operation, or zero if the event was generated when the user
;       had finished positioning the slider.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
pro AdjustLimitsMax, Event
	id_species = widget_info(Event.top, FIND_BY_UNAME='Species')
	id_minalt =  widget_info(Event.top, FIND_BY_UNAME='MinAlt')
	id_maxalt =  widget_info(Event.top, FIND_BY_UNAME='MaxAlt')
	widget_control,id_maxalt,Get_Value=maxalt
	widget_control,id_minalt,Get_Value=minalt
	if(maxalt-minalt lt 10) then begin
		minalt = maxalt-10
		widget_control,id_minalt,Set_Value=minalt
	Endif
	;print,minalt,maxalt
	return ; By Default, return the event.

end
;-----------------------------------------------------------------
; Slider Value Changed Callback Procedure.
; Argument:
;   Event structure:
;
;   {WIDGET_SLIDER, ID:0L, TOP:0L, HANDLER:0L, VALUE:0L, DRAG:0}
;
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;   VALUE returns the new value of the slider. DRAG returns integer 1
;       if the slider event was generated as part of a drag
;       operation, or zero if the event was generated when the user
;       had finished positioning the slider.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
pro AdjustLimitsMin, Event
	id_species = widget_info(Event.top, FIND_BY_UNAME='Species')
	id_minalt =  widget_info(Event.top, FIND_BY_UNAME='MinAlt')
	id_maxalt =  widget_info(Event.top, FIND_BY_UNAME='MaxAlt')
	widget_control,id_maxalt,Get_Value=maxalt
	widget_control,id_minalt,Get_Value=minalt
	if(maxalt-minalt lt 10) then begin
		maxalt = minalt+10
		widget_control,id_maxalt,Set_Value=maxalt
	Endif
	;print,minalt,maxalt
	Return
end;-----------------------------------------------------------------
; Slider Value Changed Callback Procedure.
; Argument:
;   Event structure:
;
;   {WIDGET_SLIDER, ID:0L, TOP:0L, HANDLER:0L, VALUE:0L, DRAG:0}
;
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;   VALUE returns the new value of the slider. DRAG returns integer 1
;       if the slider event was generated as part of a drag
;       operation, or zero if the event was generated when the user
;       had finished positioning the slider.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
pro AdjustRangeMax, Event
	id_species = widget_info(Event.top, FIND_BY_UNAME='Species')
	id_minval =  widget_info(Event.top, FIND_BY_UNAME='MinVal')
	id_maxval =  widget_info(Event.top, FIND_BY_UNAME='MaxVal')
	widget_control,id_maxval,Get_Value=maxval
	widget_control,id_minval,Get_Value=minval
	if(maxval-minval lt 1) then begin
		minval = maxval - 1
		widget_control,id_minval,Set_Value=minval
	Endif

end
;-----------------------------------------------------------------
; Slider Value Changed Callback Procedure.
; Argument:
;   Event structure:
;
;   {WIDGET_SLIDER, ID:0L, TOP:0L, HANDLER:0L, VALUE:0L, DRAG:0}
;
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;   VALUE returns the new value of the slider. DRAG returns integer 1
;       if the slider event was generated as part of a drag
;       operation, or zero if the event was generated when the user
;       had finished positioning the slider.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
pro AdjustRangeMin, Event
	id_species = widget_info(Event.top, FIND_BY_UNAME='Species')
	id_minval =  widget_info(Event.top, FIND_BY_UNAME='MinVal')
	id_maxval =  widget_info(Event.top, FIND_BY_UNAME='MaxVal')
	widget_control,id_maxval,Get_Value=maxval
	widget_control,id_minval,Get_Value=minval
	if(maxval-minval lt 1) then begin
		maxval = minval +1
		widget_control,id_maxval,Set_Value=maxval
	Endif
	;print,minval,maxval

end
;-----------------------------------------------------------------
; Handle Event Callback Function.
; Argument:
;   Event structure:
;
;   {WIDGET_<type>, ID:0L, TOP:0L, HANDLER:0L,...}
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;
;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
function ColorLoad, Event
     return, Event ; By Default, return the event.
end
;-----------------------------------------------------------------
; Activate Button Callback Procedure.
; Argument:
;   Event structure:
;
;   {WIDGET_BUTTON, ID:0L, TOP:0L, HANDLER:0L, SELECT:0}
;
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;   SELECT is set to 1 if the button was set, and 0 if released.
;       Normal buttons do not generate events when released, so
;       SELECT will always be 1. However, toggle buttons (created by
;       parenting a button to an exclusive or non-exclusive base)
;       return separate events for the set and release actions.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
pro SaveImage, Event
imageout = string(long(systime(/seconds) mod 100000000),$
                  format='(''S2DB_'',i8.8,''.png'')')

device,get_visual_name=vname
print,vname
if(vname eq 'TrueColor') then begin
    a = tvrd(True=1)
    WRITE_PNG,imageout,a
endif else begin
    a = tvrd()
    tvlct,r,g,b,/get
    WRITE_PNG,imageout,a,r,g,b
endelse
end
;-----------------------------------------------------------------
; Handle Event Callback Function.
; Argument:
;   Event structure:
;
;   {WIDGET_<type>, ID:0L, TOP:0L, HANDLER:0L,...}
;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;
;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
function PS_Save, Event

	id=widget_info(Event.top, FIND_BY_UNAME='PS_Save')
	widget_control,id,GET_UVALUE=pswrite
	if(pswrite eq 1) then begin
		pswrite = 0
	endif else begin
		pswrite = 1
	endelse
	widget_control,id,Set_Uvalue = pswrite
	return, Event ; By Default, return the event.

end
;---------------------------------------------------------
;function Fast
;purpose: toggle Fast check box
;---------------------------------------------------------
function Fast, Event
    id=widget_info(Event.top, FIND_BY_UNAME='Fast')
    widget_control,id,GET_UVALUE=val
    if (val eq 1) then val = 0 else val = 1
    widget_control,id,Set_Uvalue = val
    return,Event
end
;-----------------------------------------------------------------
; Post Create Widget Procedure.
; Argument:
;   wWidget - ID number of specific widget.
;
;   Any keywords passed into the generated widget creation procedure
;       are passed into this procudure.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
pro PS_Start, wWidget, _EXTRA=_VWBExtra_
	pswrite = 0
	widget_control,wWidget,Set_Uvalue = pswrite

end

pro Fast_Start, wWidget, _EXTRA=_VWBExtra_
        widget_control,wWidget,Set_Uvalue = 0
end
pro winsize, Event, buttonx=buttonx, buttony=buttony
common size,x,y
if keyword_set(buttonx) then begin
    x = buttonx
    y = buttony
endif else begin
;calculate desired draw window size
    drawx = event.x - x
    drawy = event.y < drawx*500/577.
;resize the draw window
    id_draw = widget_info(event.top, find_by_uname='Draw')
    widget_control, id_draw, scr_xsize=drawx, scr_ysize=drawy
endelse
end

pro AdjustDate, event
;decide if it was the "Prev" button or the "Next" button
if event.id eq widget_info(event.top, find_by_uname='Prev') then add = -1 else add = 1

firstday = [1,32,60,91,121,152,182,213,244,274,305,335,366]

;get the currently set date
id_year = widget_info(event.top, FIND_BY_UNAME='Year')
id_month = widget_info(event.top, find_by_uname='Month')
id_day = widget_info(event.top, find_by_uname='Day')

year = widget_info(id_year, /droplist_select)
month = widget_info(id_month, /droplist_select)
day = widget_info(id_day, /droplist_select)

;determine leap year (skipping 100 year rule w/assumption that we
;won't make it to 2100)
fullyear = year + 1984
if (fullyear mod 4 eq 0) then begin
    firstday(2:12) = firstday(2:12) + 1
endif

;calculate day since beginning of year
dsj1 = day + firstday(month)
;add or subtract 1
dsj1 = dsj1 + add
;check if it has run over the boundaries
if dsj1 lt 1 then begin
    dsj1 = firstday(12)-1
    year = year - 1
endif else if dsj1 gt firstday(12)-1 then begin
    dsj1 = 1
    year = year + 1
endif
;convert back
month = where(firstday le dsj1,nm) & month = month(nm-1)
day = dsj1 - firstday(month)

widget_control, id_year, set_droplist_select=year
widget_control, id_month, set_droplist_select=month
widget_control, id_day, set_droplist_select=day

plotdata,event

end

pro ChooseVersion, Event
;only need to do work here if "Browse" is selected
x = widget_info(event.id,/droplist_select)
n = widget_info(event.id,/droplist_number)
;use sage_filenames to get the directory, but the date is a dummy
if x eq n-1 then begin
;sage_filenames will open the browse box, and set !_versdefault system var.
    sage_filenames,spec,index,/version,yyyymm=198410
endif

end




