;+
; NAME:
;	SET_GRAPH
; PURPOSE:
;	Make setting up device dependant defaults easy
; CATEGORY:
;	?
; CALLING SEQUENCE:
;	SET_GRAPH,/device_type
; INPUTS:
;	KEYWORDS:
;		/ps	sets up for black and white postscript
;		/x	sets up for X windows
;		/cps	sets up for color postscript
;		/land	select the landscape option (used with /ps or /cps)
;		/legal	change size for 8.5"x 14" paper (used with /cps)
;		/s11x17	change size for 11" x 17" paper (used with /cps)
; OUTPUTS:
;	NONE.
; COMMON BLOCKS:
;	NONE.
; SIDE EFFECTS:
;	Changes a bunch of items that control plotting
; RESTRICTIONS:
;	NONE.  Modify to suit your personal desires.
; PROCEDURE:
;	STRAIGHTFORWARD
; MODIFICATION HISTORY:
;	May  1, 1989	J. M. Zawodny, NASA/LaRC, MS 475, Hampton VA, 23665.
;	Dec 12, 1989	JMZ, Added /LEGAL, and changed setting of !n.THICK
;-
pro SET_GPH,dummy,ps=ps,x=x,cps=cps,land=land,legal=legal,s11x17=s11x17,$
       fn=fn,vector=vector,win=win,sun=sun
   outs = ''
   if keyword_set(cps) then begin
      set_plot,'ps'
      if keyword_set(land) then begin
         xsize = 10.20      ; x size of plot area in inches
         ysize =  7.65      ; y size of plot area in inches
         xcen  =  4.25
         ycen  =  5.50
         if keyword_set(legal) then begin
            xsize = 10.50   ; x size of plot area in inches
            ysize =  8.10   ; y size of plot area in inches
            xcen  =  4.25
            ycen  =  5.50
         endif
         if keyword_set(s11x17) then begin
   ; Tested OK
            xsize = 16.25   ; x size of plot area in inches
            ysize = 10.50   ; y size of plot area in inches
;            xcen  =  5.50
;            ycen  =  8.50
            xcen  =  8.50
            ycen  =  5.50
            outs  = 'statusdict begin 11x17tray statusdict end'
         endif
         xoff  = xcen-ysize/2.   ; x offset to center the plot
         yoff  = ycen+xsize/2.   ; y offset to center the plot
         device,land=land,/bkman,/light,/color,bits=8
      endif else begin
         xsize =  7.65     ; x size of plot area in inches
         ysize = 10.20      ; y size of plot area in inches
         xcen  =  4.25
         ycen  =  5.50
         if keyword_set(legal) then begin
            xsize = 8.1   ; x size of plot area in inches
            ysize = 10.5   ; y size of plot area in inches
            xcen  = 4.25
            ycen  = 5.50
         endif
         if keyword_set(s11x17) then begin
            xsize = 10.5   ; x size of plot area in inches
            ysize = 16.25   ; y size of plot area in inches
            xcen  = 5.50
            ycen  = 8.50
            outs  = 'statusdict begin 11x17tray statusdict end'
         endif
         xoff  = xcen-xsize/2.   ; x offset to center the plot
         yoff  = ycen-ysize/2.   ; y offset to center the plot
         device,/bkman,/light,/color,bits=8
      endelse

      device,/helvetica,font_index=20
      device,/bkman,/light,font_index=3
      device,/inch,xoff=xoff,yoff=yoff,xsize=xsize,ysize=ysize
;;      if(outs ne '') then device,output=outs
      if keyword_set(fn) then device,filename=fn else device,filename='idl.ps'
      if keyword_set(vector) then !p.font = -1 else !p.font = 0
      thick   = 2
   endif

   if keyword_set(ps) then begin
      set_plot,'ps'
      if keyword_set(land) then begin
         xsize = 10.			; x size of plot area in inches
         ysize = 7.5			; y size of plot area in inches
         xoff  = 4.25-ysize/2.		; x offset to center the plot
         yoff  = 5.50+xsize/2.		; y offset to center the plot
         if keyword_set(s11x17) then begin
            xsize = 16.25		; x size of plot area in inches
            ysize = 10.50		; y size of plot area in inches
            xcen  =  5.50
            ycen  =  8.50
            xoff  = xcen-ysize/2.	; x offset to center the plot
            yoff  = ycen+xsize/2.	; y offset to center the plot
         endif
         device,land=land,/bkman,/light
      endif else begin
         xsize = 8.0			; x size of plot area in inches
         ysize = 10.			; y size of plot area in inches
         xoff  = 4.25-xsize/2.		; x offset to center the plot
         yoff  = 5.50-ysize/2.		; y offset to center the plot
         device,/bkman,/light
         if keyword_set(s11x17) then begin
            xsize = 10.5		; x size of plot area in inches
            ysize = 16.25		; y size of plot area in inches
            xcen  = 5.50
            ycen  = 8.50
            xoff  = xcen-xsize/2.	; x offset to center the plot
            yoff  = ycen-ysize/2.	; y offset to center the plot
         endif
      endelse

      device,/helvetica,font_index=20
      device,/bkman,/light,font_index=3
      device,/inch,xoff=xoff,yoff=yoff,xsize=xsize,ysize=ysize
      if keyword_set(fn) then device,filename=fn else device,filename='idl.ps'
      if keyword_set(vector) then !p.font=-1 else !p.font=0
      thick=2
   endif

   if keyword_set(x) or keyword_set(win) or keyword_set(sun) then begin
      set_plot,'x'
      !p.font = -1
      thick   = 1
   endif

   getfilename
   !p.thick = thick
   !x.thick = thick
   !y.thick = thick
return
end

