pro setshortcuts,shortcuts
;+
;subroutine to read in the data file which contain the shortcuts
;and full pathnames of SAGE II versions for a particular user.
;-

resolve_routine,'sageprof',/no_recompile
;the name of the configuration file
filename = 'sageprof.conf'
;the maximum number of shortcuts represented there
max = 100

;initialize shortcuts to zero, in case it isn't found
shortcuts = 0

;look for the file first in the local directory
file = findfile(filename,count=count)
;if not found, look in the directory that contains sageprof
if count eq 0 then begin
    str = routine_info('sageprof',/source)
;to parse the path and filename, first reverse the string
    revpath = string(reverse(byte(str.path)))
;search for the first foward or backward slash-- the one just before
;the filename in the original string
    pos = [strpos(revpath,'/'),strpos(revpath,'\')]
    gd = where(pos ne -1)
    if gd(0) ne -1 then pos = min(pos(gd)) else pos = -1
    revpath = strmid(revpath,pos,strlen(revpath)-1)
;and reverse back again to get the path
    path = string(reverse(byte(revpath)))
;now search in that directory for the configuration file
    file = findfile(path+filename,count=count)
;if it still isn't found, give up
    if count eq 0 then return
endif

;open the file
openr,lun,file,/get_lun
;initialize the input variable
lines = strarr(2)
;initialize the storage array
allpaths = strarr(max,2)
;initialize the counter 
count = 0
while not EOF(lun) do begin
    readf,lun,lines
    allpaths(count,*) = lines
    count = count + 1
endwhile
close,lun & free_lun,lun    

;initialize the output structure
shortcuts = {PATH_INFO,version:'',path:''}
shortcuts = replicate(shortcuts,count)

;copy the information into the structure
shortcuts.version = strtrim(allpaths(0:count-1,0),2)
shortcuts.path = strtrim(allpaths(0:count-1,1),2)

end


